from util import *


@responses.activate
def test_send_sms_with_messages_api(messages, dummy_data):
    stub(responses.POST, 'https://api.nexmo.com/v1/messages')

    params = {
        'channel': 'sms',
        'message_type': 'text',
        'to': '447123456789',
        'from': 'Vonage',
        'text': 'Hello from Vonage',
    }

    assert isinstance(messages.send_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert b'"from": "Vonage"' in request_body()
    assert b'"to": "447123456789"' in request_body()
    assert b'"text": "Hello from Vonage"' in request_body()


@responses.activate
def test_send_whatsapp_image_with_messages_api(messages, dummy_data):
    stub(responses.POST, 'https://api.nexmo.com/v1/messages')

    params = {
        'channel': 'whatsapp',
        'message_type': 'image',
        'to': '447123456789',
        'from': '440123456789',
        'image': {'url': 'https://example.com/image.jpg', 'caption': 'fake test image'},
    }

    assert isinstance(messages.send_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert b'"from": "440123456789"' in request_body()
    assert b'"to": "447123456789"' in request_body()
    assert (
        b'"image": {"url": "https://example.com/image.jpg", "caption": "fake test image"}'
        in request_body()
    )


@responses.activate
def test_revoke_rcs_message(messages, dummy_data):
    stub(
        responses.PATCH,
        'https://api.nexmo.com/v1/messages/abcd-ef01-2345-6789',
        'no_content.json',
    )

    assert messages.revoke_outbound_rcs_message('abcd-ef01-2345-6789') is None
    assert request_user_agent() == dummy_data.user_agent
    assert b'"status": "revoked"' in request_body()


@responses.activate
def test_mark_whatsapp_message_read(messages, dummy_data):
    stub(
        responses.PATCH,
        'https://api.nexmo.com/v1/messages/abcd-ef01-2345-6789',
        'no_content.json',
    )

    assert messages.mark_whatsapp_message_read('abcd-ef01-2345-6789') is None
    assert request_user_agent() == dummy_data.user_agent
    assert b'"status": "read"' in request_body()



from pydantic import BaseModel, Field, ValidationInfo, field_validator, constr, confloat, conint
from typing import Any, Dict,  Union, List
from typing_extensions import Annotated, Literal

from .connect_endpoints import ConnectEndpoints
from .input_types import InputTypes
from .pay_prompts import PayPrompts

from deprecated import deprecated


class Ncco:
    class Action(BaseModel):
        action: Literal['record', 'conversation', 'connect',
                        'talk', 'stream', 'input', 'notify', 'pay'] = None

    class Record(Action):
        """Use the record action to record a call or part of a call."""

        action: Literal['record'] = 'record'
        format: Literal['mp3', 'wav', 'ogg'] = None
        split: Literal['conversation'] = None
        channels: conint(ge=1, le=32) = None
        endOnSilence: conint(ge=3, le=10) = None
        endOnKey: constr(pattern='^[0-9*#]$') = None
        timeOut: conint(ge=3, le=7200) = None
        beepStart: bool = None
        eventUrl: Union[List[str], str] = None
        eventMethod: constr(to_upper=True) = None

        @field_validator('channels')
        @classmethod
        def enable_split(cls, v, info: ValidationInfo):
            values = info.data
            if values['split'] is None:
                values['split'] = 'conversation'
            return v

        @field_validator('eventUrl')
        @classmethod
        def ensure_url_in_list(cls, v):
            return Ncco._ensure_object_in_list(v)

    class Conversation(Action):
        """You can use the conversation action to create standard or moderated conferences,
        while preserving the communication context.
        Using conversation with the same name reuses the same persisted conversation."""

        action: Literal['conversation'] = 'conversation'
        name: str
        musicOnHoldUrl: Union[List[str], str] = None
        startOnEnter: bool = None
        endOnExit: bool = None
        record: bool = None
        canSpeak: List[str] = None
        canHear: List[str] = None
        mute: bool = None

        @field_validator('musicOnHoldUrl')
        @classmethod
        def ensure_url_in_list(cls, v: Any):
            return Ncco._ensure_object_in_list(v)

        @field_validator('mute')
        @classmethod
        def can_mute(cls, v, info: ValidationInfo):
            values = info.data
            if 'canSpeak' in values and values['canSpeak'] is not None:
                raise ValueError('Cannot use mute option if canSpeak option is specified.')
            return v

    class Connect(Action):
        """You can use the connect action to connect a call to endpoints such as phone numbers or a VBC extension."""

        action: Literal['connect'] = 'connect'
        endpoint: Union[dict, ConnectEndpoints.Endpoint, List]
        from_: Annotated[str, Field(alias='from_', serialization_alias='from',
                                    pattern=r'^[1-9]\d{6,14}$')] = None

        randomFromNumber: bool = None
        eventType: Literal['synchronous'] = None
        timeout: int = None
        limit: conint(le=7200) = None
        machineDetection: Literal['continue', 'hangup'] = None
        advancedMachineDetection: dict = None
        eventUrl: Union[List[str], str] = None
        eventMethod: constr(to_upper=True) = None
        ringbackTone: str = None

        @field_validator('endpoint')
        @classmethod
        def validate_endpoint(cls, v: Any):

            if type(v) is dict:
                return [ConnectEndpoints.create_endpoint_model_from_dict(v)]
            elif type(v) is list:
                return [ConnectEndpoints.create_endpoint_model_from_dict(v[0])]
            else:
                return [v]

        @field_validator('randomFromNumber')
        @classmethod
        def check_from_not_set(cls, v, info: ValidationInfo):
            values = info.data
            if v is True and 'from_' in values:
                if values['from_'] is not None:
                    raise ValueError(
                        'Cannot set a "from" ("from_") field and also the "randomFromNumber" = True option'
                    )
            return v

        @field_validator('eventUrl')
        @classmethod
        def ensure_url_in_list(cls, v):
            return Ncco._ensure_object_in_list(v)

        @field_validator('advancedMachineDetection')
        @classmethod
        def validate_advancedMachineDetection(cls, v):
            if 'behavior' in v and v['behavior'] not in ('continue', 'hangup'):
                raise ValueError(
                    'advancedMachineDetection["behavior"] must be one of: "continue", "hangup".'
                )
            if 'mode' in v and v['mode'] not in ('detect, detect_beep'):
                raise ValueError(
                    'advancedMachineDetection["mode"] must be one of: "detect", "detect_beep".'
                )
            return v

    class Talk(Action):
        """The talk action sends synthesized speech to a Conversation."""

        action: Literal['talk'] = 'talk'
        text: constr(max_length=1500)
        bargeIn: bool = None
        loop: conint(ge=0) = None
        level: confloat(ge=-1, le=1) = None
        language: str = None
        style: int = None
        premium: bool = None

    class Stream(Action):
        """The stream action allows you to send an audio stream to a Conversation."""

        action: Literal['stream'] = 'stream'
        streamUrl: Union[List[str], str]
        level: confloat(ge=-1, le=1) = None
        bargeIn: bool = None
        loop: conint(ge=0) = None

        @field_validator('streamUrl')
        @classmethod
        def ensure_url_in_list(cls, v):
            return Ncco._ensure_object_in_list(v)

    class Input(Action):
        """Collect digits or speech input by the person you are are calling."""

        action: Literal['input'] = 'input'

        type: Union[
            Literal['dtmf', 'speech'],
            List[Literal['dtmf']],
            List[Literal['speech']],
            List[Literal['dtmf', 'speech']],
        ]
        dtmf: Union[InputTypes.Dtmf, dict] = None
        speech: Union[InputTypes.Speech, dict] = None
        eventUrl: Union[List[str], str] = None
        eventMethod: constr(to_upper=True) = None

        @field_validator('type', 'eventUrl')
        @classmethod
        def ensure_value_in_list(cls, v):
            return Ncco._ensure_object_in_list(v)

        @field_validator('dtmf')
        @classmethod
        def ensure_input_object_is_dtmf_model(cls, v):
            if type(v) is dict:
                return InputTypes.create_dtmf_model(v)
            else:
                return v

        @field_validator('speech')
        @classmethod
        def ensure_input_object_is_speech_model(cls, v):
            if type(v) is dict:
                return InputTypes.create_speech_model(v)
            else:
                return v

    class Notify(Action):
        """Use the notify action to send a custom payload to your event URL."""

        action: Literal['notify'] = 'notify'

        payload: dict
        eventUrl: Union[List[str], str]
        eventMethod: constr(to_upper=True) = None

        @field_validator('eventUrl')
        @classmethod
        def ensure_url_in_list(cls, v):
            return Ncco._ensure_object_in_list(v)

    @deprecated(version='3.2.3', reason='The Pay NCCO action has been deprecated.')
    class Pay(Action):
        """The pay action collects credit card information with DTMF input in a secure (PCI-DSS compliant) way."""

        action: Literal['pay'] = 'pay'
        amount: confloat(ge=0)
        currency: constr(to_lower=True) = None
        eventUrl: Union[List[str], str] = None
        prompts: Union[List[PayPrompts.TextPrompt], PayPrompts.TextPrompt, dict] = None
        voice: Union[PayPrompts.VoicePrompt, dict] = None

        @field_validator('amount')
        @classmethod
        def round_amount(cls, v):
            return round(v, 2)

        @field_validator('eventUrl')
        @classmethod
        def ensure_url_in_list(cls, v):
            return Ncco._ensure_object_in_list(v)

        @field_validator('prompts')
        @classmethod
        def ensure_text_model(cls, v):
            if type(v) is dict:
                return PayPrompts.create_text_model(v)
            else:
                return v

        @field_validator('voice')
        @classmethod
        def ensure_voice_model(cls, v):
            if type(v) is dict:
                return PayPrompts.create_voice_model(v)
            else:
                return v

    @staticmethod
    def build_ncco(*args: Action, actions: List[Action] = None) -> str:
        ncco = []
        if actions is not None:
            for action in actions:
                ncco.append(action.model_dump(exclude_none=True, by_alias=True))
        for action in args:
            ncco.append(action.model_dump(exclude_none=True, by_alias=True))
        return ncco

    @staticmethod
    def _ensure_object_in_list(obj):
        if type(obj) != list:
            return [obj]
        else:
            return obj



from vonage.camara_auth import CamaraAuth
from util import *

import responses


@responses.activate
def test_oidc_request(camara_auth: CamaraAuth):
    stub(
        responses.POST,
        'https://api-eu.vonage.com/oauth2/bc-authorize',
        fixture_path='camara_auth/oidc_request.json',
    )

    response = camara_auth.make_oidc_request(
        number='447700900000',
        scope='dpv:FraudPreventionAndDetection#check-sim-swap',
    )

    assert response['auth_req_id'] == 'arid/8b0d35f3-4627-487c-a776-aegtdsf4rsd2'
    assert response['expires_in'] == 300
    assert response['interval'] == 0


@responses.activate
def test_request_backend_camara_access_token(camara_auth: CamaraAuth):
    stub(
        responses.POST,
        'https://api-eu.vonage.com/oauth2/token',
        fixture_path='camara_auth/token_request.json',
    )

    oidc_response = {
        'auth_req_id': 'arid/8b0d35f3-4627-487c-a776-aegtdsf4rsd2',
        'expires_in': 300,
        'interval': 0,
    }
    response = camara_auth.request_backend_camara_token(oidc_response)

    assert (
        response
        == 'eyJhbGciOiJSUzI1NiIsImprdSI6Imh0dHBzOi8vYW51YmlzLWNlcnRzLWMxLWV1dzEucHJvZC52MS52b25hZ2VuZXR3b3Jrcy5uZXQvandrcyIsImtpZCI6IkNOPVZvbmFnZSAxdmFwaWd3IEludGVybmFsIENBOjoxOTUxODQ2ODA3NDg1NTYwNjYzODY3MTM0NjE2MjU2MTU5MjU2NDkiLCJ0eXAiOiJKV1QiLCJ4NXUiOiJodHRwczovL2FudWJpcy1jZXJ0cy1jMS1ldXcxLnByb2QudjEudm9uYWdlbmV0d29ya3MubmV0L3YxL2NlcnRzLzA4NjliNDMyZTEzZmIyMzcwZTk2ZGI4YmUxMDc4MjJkIn0.eyJwcmluY2lwYWwiOnsiYXBpS2V5IjoiNGI1MmMwMGUiLCJhcHBsaWNhdGlvbklkIjoiMmJlZTViZWQtNmZlZS00ZjM2LTkxNmQtNWUzYjRjZDI1MjQzIiwibWFzdGVyQWNjb3VudElkIjoiNGI1MmMwMGUiLCJjYXBhYmlsaXRpZXMiOlsibmV0d29yay1hcGktZmVhdHVyZXMiXSwiZXh0cmFDb25maWciOnsiY2FtYXJhU3RhdGUiOiJmb0ZyQndnOFNmeGMydnd2S1o5Y3UrMlgrT0s1K2FvOWhJTTVGUGZMQ1dOeUlMTHR3WmY1dFRKbDdUc1p4QnY4QWx3aHM2bFNWcGVvVkhoWngvM3hUenFRWVkwcHpIZE5XL085ZEdRN1RKOE9sU1lDdTFYYXFEcnNFbEF4WEJVcUpGdnZTTkp5a1A5ZDBYWVN4ajZFd0F6UUFsNGluQjE1c3VMRFNsKy82U1FDa29Udnpld0tvcFRZb0F5MVg2dDJVWXdEVWFDNjZuOS9kVWxIemN3V0NGK3QwOGNReGxZVUxKZyt3T0hwV2xvWGx1MGc3REx0SCtHd0pvRGJoYnMyT2hVY3BobGZqajBpeHQ1OTRsSG5sQ1NYNkZrMmhvWEhKUW01S3JtOVBKSmttK0xTRjVsRTd3NUxtWTRvYTFXSGpkY0dwV1VsQlNQY000YnprOGU0bVE9PSJ9fSwiZmVkZXJhdGVkQXNzZXJ0aW9ucyI6e30sImF1ZCI6ImFwaS1ldS52b25hZ2UuY29tIiwiZXhwIjoxNzE3MDkyODY4LCJqdGkiOiJmNDZhYTViOC1hODA2LTRjMzctODQyMS02OGYwMzJjNDlhMWYiLCJpYXQiOjE3MTcwOTE5NzAsImlzcyI6IlZJQU0tSUFQIiwibmJmIjoxNzE3MDkxOTU1fQ.iLUbyDPR1HGLKh29fy6fqK65Q1O7mjWOletAEPJD4eu7gb0E85EL4M9R7ckJq5lIvgedQt3vBheTaON9_u-VYjMqo8ulPoEoGUDHbOzNbs4MmCW0_CRdDPGyxnUhvcbuJhPgnEHxmfHjJBljncUnk-Z7XCgyNajBNXeQQnHkRF_6NMngxJ-qjjhqbYL0VsF_JS7-TXxixNL0KAFl0SeN2DjkfwRBCclP-69CTExDjyOvouAcchqi-6ZYj_tXPCrTADuzUrQrW8C5nHp2-XjWJSFKzyvi48n8V1U6KseV-eYzBzvy7bJf0tRMX7G6gctTYq3DxdC_eXvXlnp1zx16mg'
    )


@responses.activate
def test_request_frontend_camara_access_token(camara_auth: CamaraAuth):
    stub(
        responses.POST,
        'https://api-eu.vonage.com/oauth2/token',
        fixture_path='camara_auth/token_request.json',
    )

    response = camara_auth.request_frontend_camara_token('code', 'https://example.com/callback')

    assert (
        response
        == 'eyJhbGciOiJSUzI1NiIsImprdSI6Imh0dHBzOi8vYW51YmlzLWNlcnRzLWMxLWV1dzEucHJvZC52MS52b25hZ2VuZXR3b3Jrcy5uZXQvandrcyIsImtpZCI6IkNOPVZvbmFnZSAxdmFwaWd3IEludGVybmFsIENBOjoxOTUxODQ2ODA3NDg1NTYwNjYzODY3MTM0NjE2MjU2MTU5MjU2NDkiLCJ0eXAiOiJKV1QiLCJ4NXUiOiJodHRwczovL2FudWJpcy1jZXJ0cy1jMS1ldXcxLnByb2QudjEudm9uYWdlbmV0d29ya3MubmV0L3YxL2NlcnRzLzA4NjliNDMyZTEzZmIyMzcwZTk2ZGI4YmUxMDc4MjJkIn0.eyJwcmluY2lwYWwiOnsiYXBpS2V5IjoiNGI1MmMwMGUiLCJhcHBsaWNhdGlvbklkIjoiMmJlZTViZWQtNmZlZS00ZjM2LTkxNmQtNWUzYjRjZDI1MjQzIiwibWFzdGVyQWNjb3VudElkIjoiNGI1MmMwMGUiLCJjYXBhYmlsaXRpZXMiOlsibmV0d29yay1hcGktZmVhdHVyZXMiXSwiZXh0cmFDb25maWciOnsiY2FtYXJhU3RhdGUiOiJmb0ZyQndnOFNmeGMydnd2S1o5Y3UrMlgrT0s1K2FvOWhJTTVGUGZMQ1dOeUlMTHR3WmY1dFRKbDdUc1p4QnY4QWx3aHM2bFNWcGVvVkhoWngvM3hUenFRWVkwcHpIZE5XL085ZEdRN1RKOE9sU1lDdTFYYXFEcnNFbEF4WEJVcUpGdnZTTkp5a1A5ZDBYWVN4ajZFd0F6UUFsNGluQjE1c3VMRFNsKy82U1FDa29Udnpld0tvcFRZb0F5MVg2dDJVWXdEVWFDNjZuOS9kVWxIemN3V0NGK3QwOGNReGxZVUxKZyt3T0hwV2xvWGx1MGc3REx0SCtHd0pvRGJoYnMyT2hVY3BobGZqajBpeHQ1OTRsSG5sQ1NYNkZrMmhvWEhKUW01S3JtOVBKSmttK0xTRjVsRTd3NUxtWTRvYTFXSGpkY0dwV1VsQlNQY000YnprOGU0bVE9PSJ9fSwiZmVkZXJhdGVkQXNzZXJ0aW9ucyI6e30sImF1ZCI6ImFwaS1ldS52b25hZ2UuY29tIiwiZXhwIjoxNzE3MDkyODY4LCJqdGkiOiJmNDZhYTViOC1hODA2LTRjMzctODQyMS02OGYwMzJjNDlhMWYiLCJpYXQiOjE3MTcwOTE5NzAsImlzcyI6IlZJQU0tSUFQIiwibmJmIjoxNzE3MDkxOTU1fQ.iLUbyDPR1HGLKh29fy6fqK65Q1O7mjWOletAEPJD4eu7gb0E85EL4M9R7ckJq5lIvgedQt3vBheTaON9_u-VYjMqo8ulPoEoGUDHbOzNbs4MmCW0_CRdDPGyxnUhvcbuJhPgnEHxmfHjJBljncUnk-Z7XCgyNajBNXeQQnHkRF_6NMngxJ-qjjhqbYL0VsF_JS7-TXxixNL0KAFl0SeN2DjkfwRBCclP-69CTExDjyOvouAcchqi-6ZYj_tXPCrTADuzUrQrW8C5nHp2-XjWJSFKzyvi48n8V1U6KseV-eYzBzvy7bJf0tRMX7G6gctTYq3DxdC_eXvXlnp1zx16mg'
    )


def test_get_oidc_url(camara_auth: CamaraAuth):
    url = camara_auth.get_oidc_url(
        redirect_uri='https://example.com/callback',
        state='state_id',
        login_hint='447700900000',
    )

    assert (
        url
        == 'https://oidc.idp.vonage.com/oauth2/auth?client_id=nexmo-application-id&redirect_uri=https%3A%2F%2Fexample.com%2Fcallback&response_type=code&scope=openid+dpv%3AFraudPreventionAndDetection%23number-verification-verify-read&state=state_id&login_hint=%2B447700900000'
    )



import os
import os.path
import platform

import pytest


# Ensure our client isn't being configured with real values!
os.environ.clear()


def read_file(path):
    with open(os.path.join(os.path.dirname(__file__), path)) as input_file:
        return input_file.read()


class DummyData(object):
    def __init__(self):
        from vonage import __version__

        self.api_key = "nexmo-api-key"
        self.api_secret = "nexmo-api-secret"
        self.signature_secret = "secret"
        self.application_id = "nexmo-application-id"
        self.private_key = read_file("data/private_key.txt")
        self.public_key = read_file("data/public_key.txt")
        self.user_agent = f"vonage-python/{__version__} python/{platform.python_version()}"
        self.host = "rest.nexmo.com"
        self.api_host = "api.nexmo.com"
        self.meetings_api_host = "api-eu.vonage.com/beta/meetings"


@pytest.fixture(scope="session")
def dummy_data():
    return DummyData()


@pytest.fixture
def client(dummy_data):
    from vonage.client import Client

    return Client(
        key=dummy_data.api_key,
        secret=dummy_data.api_secret,
        application_id=dummy_data.application_id,
        private_key=dummy_data.private_key,
    )


# Represents an instance of the Voice class for testing
@pytest.fixture
def voice(client):
    from vonage.voice import Voice

    return Voice(client)


# Represents an instance of the Sms class for testing
@pytest.fixture
def sms(client):
    from vonage.sms import Sms

    return Sms(client)


# Represents an instance of the Verify class for testing
@pytest.fixture
def verify(client):
    from vonage.verify import Verify

    return Verify(client)


@pytest.fixture
def number_insight(client):
    from vonage.number_insight import NumberInsight

    return NumberInsight(client)


@pytest.fixture
def account(client):
    from vonage.account import Account

    return Account(client)


@pytest.fixture
def numbers(client):
    from vonage.number_management import Numbers

    return Numbers(client)


@pytest.fixture
def ussd(client):
    from vonage.ussd import Ussd

    return Ussd(client)


@pytest.fixture
def short_codes(client):
    from vonage.short_codes import ShortCodes

    return ShortCodes(client)


@pytest.fixture
def messages(client):
    from vonage.messages import Messages

    return Messages(client)


@pytest.fixture
def redact(client):
    from vonage.redact import Redact

    return Redact(client)


@pytest.fixture
def application_v2(client):
    from vonage.application import Application

    return Application(client)


@pytest.fixture
def meetings(client):
    from vonage.meetings import Meetings

    return Meetings(client)


@pytest.fixture
def proc(client):
    from vonage.proactive_connect import ProactiveConnect

    return ProactiveConnect(client)


@pytest.fixture
def camara_auth(client):
    from vonage.camara_auth import CamaraAuth

    return CamaraAuth(client)


@pytest.fixture
def sim_swap(client):
    from vonage.sim_swap import SimSwap

    return SimSwap(client)


@pytest.fixture
def number_verification(client):
    from vonage.number_verification import NumberVerification

    return NumberVerification(client)



from vonage import Ncco, ConnectEndpoints, InputTypes, PayPrompts
import ncco_samples.ncco_action_samples as nas

import json
import pytest
from pydantic import ValidationError


def _action_as_dict(action: Ncco.Action):
    return action.model_dump(exclude_none=True, by_alias=True)


def test_record_full():
    record = Ncco.Record(
        format='wav',
        split='conversation',
        channels=4,
        endOnSilence=5,
        endOnKey='*',
        timeOut=100,
        beepStart=True,
        eventUrl=['http://example.com'],
        eventMethod='PUT',
    )
    assert type(record) == Ncco.Record
    assert json.dumps(_action_as_dict(record)) == nas.record_full


def test_record_url_passed_as_str():
    record = Ncco.Record(eventUrl='http://example.com/events')
    assert json.dumps(_action_as_dict(record)) == nas.record_url_as_str


def test_record_channels_adds_split_parameter():
    record = Ncco.Record(channels=4)
    assert json.dumps(_action_as_dict(record)) == nas.record_add_split


def test_record_model_errors():
    with pytest.raises(ValidationError):
        Ncco.Record(format='mp4')
    with pytest.raises(ValidationError):
        Ncco.Record(endOnKey='asdf')


def test_conversation_basic():
    conversation = Ncco.Conversation(name='my_conversation')
    assert type(conversation) == Ncco.Conversation
    assert json.dumps(_action_as_dict(conversation)) == nas.conversation_basic


def test_conversation_full():
    conversation = Ncco.Conversation(
        name='my_conversation',
        musicOnHoldUrl='http://example.com/music.mp3',
        startOnEnter=True,
        endOnExit=True,
        record=True,
        canSpeak=['asdf', 'qwer'],
        canHear=['asdf'],
    )
    assert json.dumps(_action_as_dict(conversation)) == nas.conversation_full


def test_conversation_field_type_error():
    with pytest.raises(ValidationError):
        Ncco.Conversation(name='my_conversation', startOnEnter='asdf')


def test_conversation_mute():
    conversation = Ncco.Conversation(name='my_conversation', mute=True)
    assert json.dumps(_action_as_dict(conversation)) == nas.conversation_mute_option


def test_conversation_incompatible_options_error():
    with pytest.raises(ValidationError) as err:
        Ncco.Conversation(name='my_conversation', canSpeak=['asdf', 'qwer'], mute=True)
    str(err.value) == 'Cannot use mute option if canSpeak option is specified.+'


def test_connect_phone_endpoint_from_dict():
    connect = Ncco.Connect(
        endpoint={
            "type": "phone",
            "number": "447000000000",
            "dtmfAnswer": "1p2p3p#**903#",
            "onAnswer": {
                "url": "https://example.com/answer",
                "ringbackTone": "http://example.com/ringbackTone.wav",
            },
        }
    )
    assert type(connect) is Ncco.Connect
    assert json.dumps(_action_as_dict(connect)) == nas.connect_phone


def test_connect_phone_endpoint_from_list():
    connect = Ncco.Connect(
        endpoint=[
            {
                "type": "phone",
                "number": "447000000000",
                "dtmfAnswer": "1p2p3p#**903#",
                "onAnswer": {
                    "url": "https://example.com/answer",
                    "ringbackTone": "http://example.com/ringbackTone.wav",
                },
            }
        ]
    )
    assert json.dumps(_action_as_dict(connect)) == nas.connect_phone


def test_connect_options():
    endpoint = ConnectEndpoints.PhoneEndpoint(number='447000000000')
    connect = Ncco.Connect(
        endpoint=endpoint,
        from_='447400000000',
        randomFromNumber=False,
        eventType='synchronous',
        timeout=15,
        limit=1000,
        machineDetection='hangup',
        eventUrl='http://example.com',
        eventMethod='PUT',
        ringbackTone='http://example.com',
    )
    assert json.dumps(_action_as_dict(connect)) == nas.connect_full


def test_connect_advanced_machine_detection():
    advancedMachineDetectionParams = {'behavior': 'continue', 'mode': 'detect'}
    endpoint = ConnectEndpoints.PhoneEndpoint(number='447000000000')
    connect = Ncco.Connect(
        endpoint=endpoint,
        from_='447400000000',
        advancedMachineDetection=advancedMachineDetectionParams,
        eventUrl='http://example.com',
    )
    assert json.dumps(_action_as_dict(connect)) == nas.connect_advancedMachineDetection


def test_connect_random_from_number_error():
    endpoint = ConnectEndpoints.PhoneEndpoint(number='447000000000')
    with pytest.raises(ValueError) as err:
        Ncco.Connect(endpoint=endpoint, from_='447400000000', randomFromNumber=True)

    assert (
        'Cannot set a "from" ("from_") field and also the "randomFromNumber" = True option'
        in str(err.value)
    )


def test_connect_validation_errors():
    endpoint = ConnectEndpoints.PhoneEndpoint(number='447000000000')
    with pytest.raises(ValidationError):
        Ncco.Connect(endpoint=endpoint, from_=1234)
    with pytest.raises(ValidationError):
        Ncco.Connect(endpoint=endpoint, eventType='asynchronous')
    with pytest.raises(ValidationError):
        Ncco.Connect(endpoint=endpoint, limit=7201)
    with pytest.raises(ValidationError):
        Ncco.Connect(endpoint=endpoint, machineDetection='do_nothing')
    with pytest.raises(ValidationError):
        Ncco.Connect(endpoint=endpoint, advancedMachineDetection={'behavior': 'do_nothing'})
    with pytest.raises(ValidationError):
        Ncco.Connect(endpoint=endpoint, advancedMachineDetection={'mode': 'detect_nothing'})


def test_talk_basic():
    talk = Ncco.Talk(text='hello')
    assert type(talk) == Ncco.Talk
    assert json.dumps(_action_as_dict(talk)) == nas.talk_basic


def test_talk_optional_params():
    talk = Ncco.Talk(
        text='hello', bargeIn=True, loop=3, level=0.5, language='en-GB', style=1, premium=True
    )
    assert json.dumps(_action_as_dict(talk)) == nas.talk_full


def test_talk_validation_error():
    with pytest.raises(ValidationError):
        Ncco.Talk(text='hello', bargeIn='go ahead')


def test_stream_basic():
    stream = Ncco.Stream(streamUrl='https://example.com/stream/music.mp3')
    assert type(stream) == Ncco.Stream
    assert json.dumps(_action_as_dict(stream)) == nas.stream_basic


def test_stream_full():
    stream = Ncco.Stream(
        streamUrl='https://example.com/stream/music.mp3', level=0.1, bargeIn=True, loop=10
    )
    assert json.dumps(_action_as_dict(stream)) == nas.stream_full


def test_input_basic():
    input = Ncco.Input(type='dtmf')
    assert type(input) == Ncco.Input
    assert json.dumps(_action_as_dict(input)) == nas.input_basic_dtmf


def test_input_basic_list():
    input = Ncco.Input(type=['dtmf', 'speech'])
    assert json.dumps(_action_as_dict(input)) == nas.input_basic_dtmf_speech


def test_input_dtmf_and_speech_options():
    dtmf = InputTypes.Dtmf(timeOut=5, maxDigits=12, submitOnHash=True)
    speech = InputTypes.Speech(
        uuid='my-uuid',
        endOnSilence=2.5,
        language='en-GB',
        context=['sales', 'billing'],
        startTimeout=20,
        maxDuration=30,
        saveAudio=True,
    )
    input = Ncco.Input(
        type=['dtmf', 'speech'],
        dtmf=dtmf,
        speech=speech,
        eventUrl='http://example.com/speech',
        eventMethod='put',
    )
    assert json.dumps(_action_as_dict(input)) == nas.input_dtmf_and_speech_full


def test_input_validation_error():
    with pytest.raises(ValidationError):
        Ncco.Input(type='invalid_type')


def test_notify_basic():
    notify = Ncco.Notify(payload={'message': 'hello'}, eventUrl=['http://example.com'])
    assert type(notify) == Ncco.Notify
    assert json.dumps(_action_as_dict(notify)) == nas.notify_basic


def test_notify_basic_str_in_event_url():
    notify = Ncco.Notify(payload={'message': 'hello'}, eventUrl='http://example.com')
    assert type(notify) == Ncco.Notify
    assert json.dumps(_action_as_dict(notify)) == nas.notify_basic


def test_notify_full():
    notify = Ncco.Notify(
        payload={'message': 'hello'}, eventUrl=['http://example.com'], eventMethod='POST'
    )
    assert type(notify) == Ncco.Notify
    assert json.dumps(_action_as_dict(notify)) == nas.notify_full


def test_notify_validation_error():
    with pytest.raises(ValidationError):
        Ncco.Notify(payload={'message', 'hello'}, eventUrl=['http://example.com'])


def test_pay_voice_basic():
    with pytest.deprecated_call():
        pay = Ncco.Pay(amount='10.00')
        assert type(pay) == Ncco.Pay
        assert json.dumps(_action_as_dict(pay)) == nas.pay_basic


def test_pay_voice_full():
    voice_settings = PayPrompts.VoicePrompt(language='en-GB', style=1)
    with pytest.deprecated_call():
        pay = Ncco.Pay(
            amount=99.99, currency='gbp', eventUrl='https://example.com/payment', voice=voice_settings
        )
        assert json.dumps(_action_as_dict(pay)) == nas.pay_voice_full


def test_pay_text():
    text_prompts = PayPrompts.TextPrompt(
        type='CardNumber',
        text='Enter your card number.',
        errors={
            'InvalidCardType': {
                'text': 'The card you are trying to use is not valid for this purchase.'
            }
        },
    )
    with pytest.deprecated_call():
        pay = Ncco.Pay(
            amount=12.345, currency='gbp', eventUrl='https://example.com/payment', prompts=text_prompts
        )
        assert json.dumps(_action_as_dict(pay)) == nas.pay_text


def test_pay_text_multiple_prompts():
    card_prompt = PayPrompts.TextPrompt(
        type='CardNumber',
        text='Enter your card number.',
        errors={
            'InvalidCardType': {
                'text': 'The card you are trying to use is not valid for this purchase.'
            }
        },
    )
    expiration_date_prompt = PayPrompts.TextPrompt(
        type='ExpirationDate',
        text='Enter your card expiration date.',
        errors={
            'InvalidExpirationDate': {'text': 'You have entered an invalid expiration date.'},
            'Timeout': {'text': 'Please enter your card\'s expiration date.'},
        },
    )
    security_code_prompt = PayPrompts.TextPrompt(
        type='SecurityCode',
        text='Enter your 3-digit security code.',
        errors={
            'InvalidSecurityCode': {'text': 'You have entered an invalid security code.'},
            'Timeout': {'text': 'Please enter your card\'s security code.'},
        },
    )

    text_prompts = [card_prompt, expiration_date_prompt, security_code_prompt]
    with pytest.deprecated_call():
        pay = Ncco.Pay(amount=12, prompts=text_prompts)
        assert json.dumps(_action_as_dict(pay)) == nas.pay_text_multiple_prompts


def test_pay_validation_error():
    with pytest.raises(ValidationError):
        with pytest.deprecated_call():
            Ncco.Pay(amount='not-valid')



from util import *


@responses.activate
def test_send_ussd_push_message(ussd, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/ussd/json")

    params = {"from": "MyCompany20", "to": "447525856424", "text": "Hello"}

    assert isinstance(ussd.send_ussd_push_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "from=MyCompany20" in request_body()
    assert "to=447525856424" in request_body()
    assert "text=Hello" in request_body()


@responses.activate
def test_send_ussd_prompt_message(ussd, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/ussd-prompt/json")

    params = {"from": "long-virtual-number", "to": "447525856424", "text": "Hello"}

    assert isinstance(ussd.send_ussd_prompt_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "from=long-virtual-number" in request_body()
    assert "to=447525856424" in request_body()
    assert "text=Hello" in request_body()



import platform

from util import *

import vonage
from vonage.errors import PricingTypeError


@responses.activate
def test_get_balance(account, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/account/get-balance")

    assert isinstance(account.get_balance(), dict)
    assert request_user_agent() == dummy_data.user_agent


@responses.activate
def test_application_info_options(dummy_data):
    app_name, app_version = "ExampleApp", "X.Y.Z"

    stub(responses.GET, "https://rest.nexmo.com/account/get-balance")

    client = vonage.Client(
        key=dummy_data.api_key,
        secret=dummy_data.api_secret,
        app_name=app_name,
        app_version=app_version,
    )
    user_agent = f"vonage-python/{vonage.__version__} python/{platform.python_version()} {app_name}/{app_version}"

    account = client.account
    assert isinstance(account.get_balance(), dict)
    assert request_user_agent() == user_agent


@responses.activate
def test_get_country_pricing(account, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/account/get-pricing/outbound/sms")

    assert isinstance(account.get_country_pricing("GB"), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "country=GB" in request_query()


@responses.activate
def test_get_all_countries_pricing(account, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/account/get-full-pricing/outbound/sms")

    assert isinstance(account.get_all_countries_pricing(), dict)
    assert request_user_agent() == dummy_data.user_agent


@responses.activate
def test_get_prefix_pricing(account, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/account/get-prefix-pricing/outbound/sms")

    assert isinstance(account.get_prefix_pricing(44), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "prefix=44" in request_query()


@responses.activate
def test_get_sms_pricing(account, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/account/get-phone-pricing/outbound/sms")

    assert isinstance(account.get_sms_pricing("447525856424"), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "phone=447525856424" in request_query()


@responses.activate
def test_get_voice_pricing(account, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/account/get-phone-pricing/outbound/voice")

    assert isinstance(account.get_voice_pricing("447525856424"), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "phone=447525856424" in request_query()


def test_invalid_pricing_type_throws_error(account):
    with pytest.raises(PricingTypeError):
        account.get_country_pricing('GB', 'not_a_valid_pricing_type')


@responses.activate
def test_update_default_sms_webhook(account, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/account/settings")

    params = {"moCallBackUrl": "http://example.com/callback"}

    assert isinstance(account.update_default_sms_webhook(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "moCallBackUrl=http%3A%2F%2Fexample.com%2Fcallback" in request_body()


@responses.activate
def test_topup(account, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/account/top-up")

    params = {"trx": "00X123456Y7890123Z"}

    assert isinstance(account.topup(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "trx=00X123456Y7890123Z" in request_body()


@responses.activate
def test_list_secrets(account):
    stub(
        responses.GET,
        "https://api.nexmo.com/accounts/myaccountid/secrets",
        fixture_path="account/secret_management/list.json",
    )

    secrets = account.list_secrets("myaccountid")
    assert_basic_auth()
    assert secrets["_embedded"]["secrets"][0]["id"] == "ad6dc56f-07b5-46e1-a527-85530e625800"


@responses.activate
def test_list_secrets_missing(account):
    stub(
        responses.GET,
        "https://api.nexmo.com/accounts/myaccountid/secrets",
        status_code=404,
        fixture_path="account/secret_management/missing.json",
    )

    with pytest.raises(vonage.ClientError) as ce:
        account.list_secrets("myaccountid")
    assert_basic_auth()
    assert (
        str(ce.value)
        == """Invalid API Key: API key 'ABC123' does not exist, or you do not have access (https://developer.nexmo.com/api-errors#invalid-api-key)"""
    )


@responses.activate
def test_get_secret(account):
    stub(
        responses.GET,
        "https://api.nexmo.com/accounts/meaccountid/secrets/mahsecret",
        fixture_path="account/secret_management/get.json",
    )

    secret = account.get_secret("meaccountid", "mahsecret")
    assert_basic_auth()
    assert secret["id"] == "ad6dc56f-07b5-46e1-a527-85530e625800"


@responses.activate
def test_create_secret(account):
    stub(
        responses.POST,
        "https://api.nexmo.com/accounts/meaccountid/secrets",
        fixture_path="account/secret_management/create.json",
    )

    secret = account.create_secret("meaccountid", "mahsecret")
    assert_basic_auth()
    assert secret["id"] == "ad6dc56f-07b5-46e1-a527-85530e625800"


@responses.activate
def test_create_secret_max_secrets(account):
    stub(
        responses.POST,
        "https://api.nexmo.com/accounts/meaccountid/secrets",
        status_code=403,
        fixture_path="account/secret_management/max-secrets.json",
    )

    with pytest.raises(vonage.ClientError) as ce:
        account.create_secret("meaccountid", "mahsecret")
    assert_basic_auth()
    assert (
        str(ce.value)
        == """Maxmimum number of secrets already met: This account has reached maximum number of '2' allowed secrets (https://developer.nexmo.com/api-errors/account/secret-management#maximum-secrets-allowed)"""
    )


@responses.activate
def test_create_secret_validation(account):
    stub(
        responses.POST,
        "https://api.nexmo.com/accounts/meaccountid/secrets",
        status_code=400,
        fixture_path="account/secret_management/create-validation.json",
    )

    with pytest.raises(vonage.ClientError) as ce:
        account.create_secret("meaccountid", "mahsecret")
    assert_basic_auth()
    assert (
        str(ce.value)
        == """Bad Request: The request failed due to validation errors (https://developer.nexmo.com/api-errors/account/secret-management#validation)"""
    )


@responses.activate
def test_delete_secret(account):
    stub(responses.DELETE, "https://api.nexmo.com/accounts/meaccountid/secrets/mahsecret")

    account.revoke_secret("meaccountid", "mahsecret")
    assert_basic_auth()


@responses.activate
def test_delete_secret_last_secret(account):
    stub(
        responses.DELETE,
        "https://api.nexmo.com/accounts/meaccountid/secrets/mahsecret",
        status_code=403,
        fixture_path="account/secret_management/last-secret.json",
    )
    with pytest.raises(vonage.ClientError) as ce:
        account.revoke_secret("meaccountid", "mahsecret")
    assert_basic_auth()
    assert (
        str(ce.value)
        == """Secret Deletion Forbidden: Can not delete the last secret. The account must always have at least 1 secret active at any time (https://developer.nexmo.com/api-errors/account/secret-management#delete-last-secret)"""
    )



from vonage import ConnectEndpoints, Ncco
import ncco_samples.ncco_action_samples as nas

import json
import pytest
from pydantic import ValidationError


def _action_as_dict(action: Ncco.Action):
    return action.model_dump(exclude_none=True)


def test_connect_all_endpoints_from_model():
    phone = ConnectEndpoints.PhoneEndpoint(
        number='447000000000',
        dtmfAnswer='1p2p3p#**903#',
        onAnswer={
            "url": "https://example.com/answer",
            "ringbackTone": "http://example.com/ringbackTone.wav",
        },
    )
    connect_phone = Ncco.Connect(endpoint=phone)
    assert json.dumps(_action_as_dict(connect_phone)) == nas.connect_phone

    app = ConnectEndpoints.AppEndpoint(user='test_user')
    connect_app = Ncco.Connect(endpoint=app)
    assert json.dumps(_action_as_dict(connect_app)) == nas.connect_app

    websocket = ConnectEndpoints.WebsocketEndpoint(
        uri='ws://example.com/socket',
        contentType='audio/l16;rate=8000',
        headers={"language": "en-GB"},
    )
    connect_websocket = Ncco.Connect(endpoint=websocket)
    assert json.dumps(_action_as_dict(connect_websocket)) == nas.connect_websocket

    sip = ConnectEndpoints.SipEndpoint(
        uri='sip:rebekka@sip.mcrussell.com',
        headers={"location": "New York City", "occupation": "developer"},
    )
    connect_sip = Ncco.Connect(endpoint=sip)
    assert json.dumps(_action_as_dict(connect_sip)) == nas.connect_sip

    vbc = ConnectEndpoints.VbcEndpoint(extension='111')
    connect_vbc = Ncco.Connect(endpoint=vbc)
    assert json.dumps(_action_as_dict(connect_vbc)) == nas.connect_vbc


def test_connect_endpoints_errors():
    with pytest.raises(ValidationError) as err:
        ConnectEndpoints.PhoneEndpoint(number='447000000000', onAnswer={'url': 'not-a-valid-url'})

    with pytest.raises(ValidationError) as err:
        ConnectEndpoints.PhoneEndpoint(
            number='447000000000',
            onAnswer={'url': 'http://example.com/answer', 'ringbackTone': 'not-a-valid-url'},
        )

    with pytest.raises(ValueError) as err:
        ConnectEndpoints.create_endpoint_model_from_dict({'type': 'carrier_pigeon'})
    assert (
        str(err.value)
        == 'Invalid "type" specified for endpoint object. Cannot create a ConnectEndpoints.Endpoint model.'
    )



from __future__ import annotations
from typing import TYPE_CHECKING
from urllib.parse import urlencode, urlunparse


if TYPE_CHECKING:
    from vonage import Client


class CamaraAuth:
    """Class containing methods for authenticating APIs following Camara standards."""

    def __init__(self, client: Client):
        self._client = client
        self._auth_type = 'jwt'

    def make_oidc_request(self, number: str, scope: str):
        """Make an OIDC request to authenticate a user.

        Returns a code that can be used to request a Camara token."""

        login_hint = f'tel:+{number}'
        params = {'login_hint': login_hint, 'scope': scope}

        return self._client.post(
            'api-eu.vonage.com',
            '/oauth2/bc-authorize',
            params=params,
            auth_type=self._auth_type,
            body_is_json=False,
        )

    def get_oidc_url(
        self,
        redirect_uri: str,
        state: str = None,
        login_hint: str = None,
        scope: str = 'openid dpv:FraudPreventionAndDetection#number-verification-verify-read',
    ):
        """Get the URL to use for authentication in a front-end application.

        Args:
            redirect_uri (str): The URI to redirect to after authentication.
            scope (str): The scope of the request.
            state (str): A unique identifier for the request. Can be any string.
            login_hint (str): The phone number to use for the request.

        Returns:
            The URL to use to make an OIDC request in a front-end application.
        """
        base_url = 'https://oidc.idp.vonage.com/oauth2/auth'

        params = {
            'client_id': self._client.application_id,
            'redirect_uri': redirect_uri,
            'response_type': 'code',
            'scope': scope,
        }
        if state:
            params['state'] = state
        if login_hint:
            if login_hint.startswith('+'):
                params['login_hint'] = login_hint
            else:
                params['login_hint'] = f'+{login_hint}'

        full_url = urlunparse(('', '', base_url, '', urlencode(params), ''))
        return full_url

    def request_backend_camara_token(
        self, oidc_response: dict, grant_type: str = 'urn:openid:params:grant-type:ciba'
    ):
        """Request a Camara token using an authentication request ID given as a
        response to the OIDC request.
        """
        params = {
            'grant_type': grant_type,
            'auth_req_id': oidc_response['auth_req_id'],
        }
        return self._request_camara_token(params)

    def request_frontend_camara_token(self, code: str, redirect_uri: str):
        """Request a Camara token using a code from an OIDC response."""
        params = {
            'grant_type': 'authorization_code',
            'code': code,
            'redirect_uri': redirect_uri,
        }
        return self._request_camara_token(params)

    def _request_camara_token(self, params: dict):
        token_response = self._client.post(
            'api-eu.vonage.com',
            '/oauth2/token',
            params=params,
            auth_type=self._auth_type,
            body_is_json=False,
        )
        return token_response['access_token']



import vonage
from util import *


@responses.activate
def test_send_message(sms, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/sms/json")

    params = {"from": "Python", "to": "447525856424", "text": "Hey!"}

    assert isinstance(sms.send_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "from=Python" in request_body()
    assert "to=447525856424" in request_body()
    assert "text=Hey%21" in request_body()


@responses.activate
def test_authentication_error(sms):
    responses.add(responses.POST, "https://rest.nexmo.com/sms/json", status=401)

    with pytest.raises(vonage.AuthenticationError):
        sms.send_message({})


@responses.activate
def test_client_error(sms):
    responses.add(responses.POST, "https://rest.nexmo.com/sms/json", status=400)

    with pytest.raises(vonage.ClientError) as excinfo:
        sms.send_message({})
    excinfo.match(r"400 response from rest.nexmo.com")


@responses.activate
def test_server_error(sms):
    responses.add(responses.POST, "https://rest.nexmo.com/sms/json", status=500)

    with pytest.raises(vonage.ServerError) as excinfo:
        sms.send_message({})
    excinfo.match(r"500 response from rest.nexmo.com")


@responses.activate
def test_submit_sms_conversion(sms):
    responses.add(responses.POST, "https://api.nexmo.com/conversions/sms", status=200, body=b"OK")

    sms.submit_sms_conversion("a-message-id")
    assert "message-id=a-message-id" in request_body()
    assert "timestamp" in request_body()



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from vonage import Client

from ._internal import set_auth_type
from .errors import MessagesError

import re


class Messages:
    valid_message_channels = {
        'sms',
        'mms',
        'rcs',
        'whatsapp',
        'messenger',
        'viber_service',
    }
    valid_message_types = {
        'sms': {'text'},
        'mms': {'image', 'vcard', 'audio', 'video'},
        'rcs': {'text', 'image', 'video', 'file', 'custom'},
        'whatsapp': {
            'text',
            'image',
            'audio',
            'video',
            'file',
            'template',
            'sticker',
            'custom',
        },
        'messenger': {'text', 'image', 'audio', 'video', 'file'},
        'viber_service': {'text', 'image', 'video', 'file'},
    }

    def __init__(self, client: Client):
        self._client = client
        self._auth_type = set_auth_type(self._client)

    def send_message(self, params: dict):
        self.validate_send_message_input(params)

        return self._client.post(
            self._client.api_host(),
            '/v1/messages',
            params,
            auth_type=self._auth_type,
        )

    def revoke_outbound_rcs_message(self, message_uuid: str) -> None:
        """Revoke an outbound RCS message.

        Note: to use this method, update the `api_host` attribute of the `vonage.Client` instance
        to the API endpoint corresponding to the region where the WhatsApp number is hosted.

        For example, to use the EU API endpoint, set the `api_host`
        attribute to 'api-eu.vonage.com'.

        Args:
            message_uuid (str): The UUID of the message to revoke.
        """
        return self._client.patch(
            self._client.api_host(),
            f'/v1/messages/{message_uuid}',
            params={'status': 'revoked'},
            auth_type=self._auth_type,
        )

    def mark_whatsapp_message_read(self, message_uuid: str) -> None:
        """Mark a WhatsApp message as read.

        Note: to use this method, update the `api_host` attribute of the `vonage.Client` instance
        to the API endpoint corresponding to the region where the WhatsApp number is hosted.

        For example, to use the EU API endpoint, set the `api_host`
        attribute to 'api-eu.vonage.com'.

        Args:
            message_uuid (str): The UUID of the message to mark as read.
        """
        return self._client.patch(
            self._client.api_host(),
            f'/v1/messages/{message_uuid}',
            params={'status': 'read'},
            auth_type=self._auth_type,
        )

    def validate_send_message_input(self, params):
        self._check_input_is_dict(params)
        self._check_valid_message_channel(params)
        self._check_valid_message_type(params)
        self._check_valid_recipient(params)
        self._check_valid_sender(params)
        self._channel_specific_checks(params)
        self._check_valid_client_ref(params)

    def _check_input_is_dict(self, params):
        if type(params) is not dict:
            raise MessagesError(
                'Parameters to the send_message method must be specified as a dictionary.'
            )

    def _check_valid_message_channel(self, params):
        if params['channel'] not in Messages.valid_message_channels:
            raise MessagesError(
                f"""
            "{params['channel']}" is an invalid message channel.
            Must be one of the following types: {self.valid_message_channels}'
            """
            )

    def _check_valid_message_type(self, params):
        if params['message_type'] not in self.valid_message_types[params['channel']]:
            raise MessagesError(
                f"""
                "{params['message_type']}" is not a valid message type for channel "{params["channel"]}".
                Must be one of the following types: {self.valid_message_types[params["channel"]]}
            """
            )

    def _check_valid_recipient(self, params):
        if not isinstance(params['to'], str):
            raise MessagesError(f'Message recipient ("to={params["to"]}") not in a valid format.')
        elif params['channel'] != 'messenger' and not re.search(r'^[1-9]\d{6,14}$', params['to']):
            raise MessagesError(
                f'Message recipient number ("to={params["to"]}") not in a valid format.'
            )
        elif params['channel'] == 'messenger' and not 0 < len(params['to']) < 50:
            raise MessagesError(
                f'Message recipient ID ("to={params["to"]}") not in a valid format.'
            )

    def _check_valid_sender(self, params):
        if not isinstance(params['from'], str) or params['from'] == "":
            raise MessagesError(
                f'Message sender ("frm={params["from"]}") set incorrectly. Set a valid name or number for the sender.'
            )

    def _channel_specific_checks(self, params):
        if (
            (
                params['channel'] == 'whatsapp'
                and params['message_type'] == 'template'
                and 'whatsapp' not in params
            )
            or (
                params['channel'] == 'whatsapp'
                and params['message_type'] == 'sticker'
                and 'sticker' not in params
            )
            or (params['channel'] == 'viber_service' and 'viber_service' not in params)
        ):
            raise MessagesError(
                f'''You must specify all required properties for message channel "{params["channel"]}".'''
            )
        elif params['channel'] == 'whatsapp' and params['message_type'] == 'sticker':
            self._check_valid_whatsapp_sticker(params['sticker'])

    def _check_valid_client_ref(self, params):
        if 'client_ref' in params:
            if len(params['client_ref']) <= 100:
                self._client_ref = params['client_ref']
            else:
                raise MessagesError('client_ref can be a maximum of 100 characters.')

    def _check_valid_whatsapp_sticker(self, sticker):
        if ('id' not in sticker and 'url' not in sticker) or ('id' in sticker and 'url' in sticker):
            raise MessagesError(
                'Must specify one, and only one, of "id" or "url" in the "sticker" field.'
            )



from vonage import PayPrompts

import pytest
from pydantic import ValidationError


def test_create_voice_model():
    voice_prompt = PayPrompts.VoicePrompt(language='en-GB', style=1)
    assert (type(voice_prompt)) == PayPrompts.VoicePrompt


def test_create_voice_model_from_dict():
    voice_dict = {'language': 'en-GB', 'style': 1}
    voice_prompt = PayPrompts.create_voice_model(voice_dict)
    assert (type(voice_prompt)) == PayPrompts.VoicePrompt


def test_create_text_model():
    text_prompt = PayPrompts.TextPrompt(
        type='CardNumber',
        text='Enter your card number.',
        errors={
            'InvalidCardType': {
                'text': 'The card you are trying to use is not valid for this purchase.'
            }
        },
    )
    assert type(text_prompt) == PayPrompts.TextPrompt


def test_create_text_model_from_dict():
    text_dict = {
        'type': 'CardNumber',
        'text': 'Enter your card number.',
        'errors': {
            'InvalidCardType': {
                'text': 'The card you are trying to use is not valid for this purchase.'
            }
        },
    }
    text_prompt = PayPrompts.create_text_model(text_dict)
    assert type(text_prompt) == PayPrompts.TextPrompt


def test_error_message_not_in_subdictionary():
    with pytest.raises(ValidationError):
        PayPrompts.TextPrompt(
            type='CardNumber',
            text='Enter your card number.',
            errors={
                'InvalidCardType': 'The card you are trying to use is not valid for this purchase.'
            },
        )


def test_invalid_error_type_for_prompt():
    with pytest.raises(ValueError) as err:
        PayPrompts.TextPrompt(
            type='SecurityCode',
            text='Enter your card number.',
            errors={
                'InvalidCardType': {
                    'text': 'The card you are trying to use is not valid for this purchase.'
                }
            },
        )

    assert (
        'Value "InvalidCardType" is not a valid error for the "SecurityCode" prompt type.'
        in str(err.value)
    )



from .client import *
from .ncco_builder.ncco import *

__version__ = "3.18.0"

from warnings import warn

warn(
    'v3 of the Vonage Python SDK is now deprecated. Please upgrade to vonage>=4.0.0.',
    DeprecationWarning,
    stacklevel=2,
)



from vonage.errors import NumberVerificationError
from vonage.number_verification import NumberVerification
from util import *

import responses
from pytest import raises


def test_get_oidc_url(number_verification: NumberVerification):
    url = number_verification.get_oidc_url(
        redirect_uri='https://example.com/callback',
        state='state_id',
        login_hint='447700900000',
    )

    assert (
        url
        == 'https://oidc.idp.vonage.com/oauth2/auth?client_id=nexmo-application-id&redirect_uri=https%3A%2F%2Fexample.com%2Fcallback&response_type=code&scope=openid+dpv%3AFraudPreventionAndDetection%23number-verification-verify-read&state=state_id&login_hint=%2B447700900000'
    )


@responses.activate
def test_create_camara_token_and_verify_numbers(number_verification: NumberVerification):
    stub(
        responses.POST,
        'https://api-eu.vonage.com/oauth2/token',
        fixture_path='camara_auth/token_request.json',
    )

    access_token = number_verification.exchange_code_for_token(
        'code', 'https://example.com/callback'
    )
    assert (
        access_token
        == 'eyJhbGciOiJSUzI1NiIsImprdSI6Imh0dHBzOi8vYW51YmlzLWNlcnRzLWMxLWV1dzEucHJvZC52MS52b25hZ2VuZXR3b3Jrcy5uZXQvandrcyIsImtpZCI6IkNOPVZvbmFnZSAxdmFwaWd3IEludGVybmFsIENBOjoxOTUxODQ2ODA3NDg1NTYwNjYzODY3MTM0NjE2MjU2MTU5MjU2NDkiLCJ0eXAiOiJKV1QiLCJ4NXUiOiJodHRwczovL2FudWJpcy1jZXJ0cy1jMS1ldXcxLnByb2QudjEudm9uYWdlbmV0d29ya3MubmV0L3YxL2NlcnRzLzA4NjliNDMyZTEzZmIyMzcwZTk2ZGI4YmUxMDc4MjJkIn0.eyJwcmluY2lwYWwiOnsiYXBpS2V5IjoiNGI1MmMwMGUiLCJhcHBsaWNhdGlvbklkIjoiMmJlZTViZWQtNmZlZS00ZjM2LTkxNmQtNWUzYjRjZDI1MjQzIiwibWFzdGVyQWNjb3VudElkIjoiNGI1MmMwMGUiLCJjYXBhYmlsaXRpZXMiOlsibmV0d29yay1hcGktZmVhdHVyZXMiXSwiZXh0cmFDb25maWciOnsiY2FtYXJhU3RhdGUiOiJmb0ZyQndnOFNmeGMydnd2S1o5Y3UrMlgrT0s1K2FvOWhJTTVGUGZMQ1dOeUlMTHR3WmY1dFRKbDdUc1p4QnY4QWx3aHM2bFNWcGVvVkhoWngvM3hUenFRWVkwcHpIZE5XL085ZEdRN1RKOE9sU1lDdTFYYXFEcnNFbEF4WEJVcUpGdnZTTkp5a1A5ZDBYWVN4ajZFd0F6UUFsNGluQjE1c3VMRFNsKy82U1FDa29Udnpld0tvcFRZb0F5MVg2dDJVWXdEVWFDNjZuOS9kVWxIemN3V0NGK3QwOGNReGxZVUxKZyt3T0hwV2xvWGx1MGc3REx0SCtHd0pvRGJoYnMyT2hVY3BobGZqajBpeHQ1OTRsSG5sQ1NYNkZrMmhvWEhKUW01S3JtOVBKSmttK0xTRjVsRTd3NUxtWTRvYTFXSGpkY0dwV1VsQlNQY000YnprOGU0bVE9PSJ9fSwiZmVkZXJhdGVkQXNzZXJ0aW9ucyI6e30sImF1ZCI6ImFwaS1ldS52b25hZ2UuY29tIiwiZXhwIjoxNzE3MDkyODY4LCJqdGkiOiJmNDZhYTViOC1hODA2LTRjMzctODQyMS02OGYwMzJjNDlhMWYiLCJpYXQiOjE3MTcwOTE5NzAsImlzcyI6IlZJQU0tSUFQIiwibmJmIjoxNzE3MDkxOTU1fQ.iLUbyDPR1HGLKh29fy6fqK65Q1O7mjWOletAEPJD4eu7gb0E85EL4M9R7ckJq5lIvgedQt3vBheTaON9_u-VYjMqo8ulPoEoGUDHbOzNbs4MmCW0_CRdDPGyxnUhvcbuJhPgnEHxmfHjJBljncUnk-Z7XCgyNajBNXeQQnHkRF_6NMngxJ-qjjhqbYL0VsF_JS7-TXxixNL0KAFl0SeN2DjkfwRBCclP-69CTExDjyOvouAcchqi-6ZYj_tXPCrTADuzUrQrW8C5nHp2-XjWJSFKzyvi48n8V1U6KseV-eYzBzvy7bJf0tRMX7G6gctTYq3DxdC_eXvXlnp1zx16mg'
    )

    stub(
        responses.POST,
        'https://api-eu.vonage.com/camara/number-verification/v031/verify',
        fixture_path='number_verification/verify.json',
    )
    response = number_verification.verify(access_token, phone_number='447700900000')
    assert response['devicePhoneNumberVerified'] == True

    new_access_token = number_verification.exchange_code_for_token(
        'code', 'https://example.com/callback'
    )
    response = number_verification.verify(new_access_token, hashed_phone_number='hash')
    assert response['devicePhoneNumberVerified'] == True


def test_verify_error_if_both_phone_number_and_hashed_phone_number_provided(
    number_verification: NumberVerification,
):
    with raises(
        NumberVerificationError,
        match='Only one of "phone_number" and "hashed_phone_number" can be provided.',
    ):
        number_verification.verify(
            access_token='asdf', phone_number='447700900000', hashed_phone_number='hash'
        )



from vonage import InputTypes


def test_create_dtmf_model():
    dtmf = InputTypes.Dtmf(timeOut=5, maxDigits=2, submitOnHash=True)
    assert type(dtmf) == InputTypes.Dtmf
    assert dtmf.model_dump() == {'maxDigits': 2, 'submitOnHash': True, 'timeOut': 5}


def test_create_dtmf_model_from_dict():
    dtmf_dict = {'timeOut': 3, 'maxDigits': 4, 'submitOnHash': True}
    dtmf_model = InputTypes.create_dtmf_model(dtmf_dict)
    assert type(dtmf_model) == InputTypes.Dtmf
    assert dtmf_model.model_dump() == {'maxDigits': 4, 'submitOnHash': True, 'timeOut': 3}


def test_create_speech_model():
    speech = InputTypes.Speech(
        uuid='my-uuid',
        endOnSilence=2.5,
        language='en-GB',
        context=['sales', 'billing'],
        startTimeout=20,
        maxDuration=30,
        saveAudio=True,
    )
    assert type(speech) == InputTypes.Speech
    assert speech.model_dump() == {
        'uuid': 'my-uuid',
        'endOnSilence': 2.5,
        'language': 'en-GB',
        'context': ['sales', 'billing'],
        'startTimeout': 20,
        'maxDuration': 30,
        'saveAudio': True,
    }


def test_create_speech_model_from_dict():
    speech_dict = {'uuid': 'my-uuid', 'endOnSilence': 2.5, 'maxDuration': 30}
    speech_model = InputTypes.create_speech_model(speech_dict)
    assert type(speech_model) == InputTypes.Speech
    assert speech_model.model_dump(exclude_none=True) == {
        'uuid': 'my-uuid',
        'endOnSilence': 2.5,
        'maxDuration': 30,
    }



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from vonage import Client


def _format_date_param(params, key, format="%Y-%m-%d %H:%M:%S"):
    """
    Utility function to convert datetime values to strings.

    If the value is already a str, or is not in the dict, no change is made.

    :param params: A `dict` of params that may contain a `datetime` value.
    :param key: The datetime value to be converted to a `str`
    :param format: The `strftime` format to be used to format the date. The default value is '%Y-%m-%d %H:%M:%S'
    """
    if key in params:
        param = params[key]
        if hasattr(param, "strftime"):
            params[key] = param.strftime(format)


def set_auth_type(client: Client) -> str:
    """Sets the authentication type used. If a JWT Client has been created,
    it will create a JWT and use JWT authentication."""

    if hasattr(client, '_jwt_client'):
        return 'jwt'
    else:
        return 'header'



from .errors import CallbackRequiredError


class NumberInsight:
    auth_type = 'params'

    def __init__(self, client):
        self._client = client

    def get_basic_number_insight(self, params=None, **kwargs):
        return self._client.get(
            self._client.api_host(),
            "/ni/basic/json",
            params or kwargs,
            auth_type=NumberInsight.auth_type,
        )

    def get_standard_number_insight(self, params=None, **kwargs):
        return self._client.get(
            self._client.api_host(),
            "/ni/standard/json",
            params or kwargs,
            auth_type=NumberInsight.auth_type,
        )

    def get_advanced_number_insight(self, params=None, **kwargs):
        return self._client.get(
            self._client.api_host(),
            "/ni/advanced/json",
            params or kwargs,
            auth_type=NumberInsight.auth_type,
        )

    def get_async_advanced_number_insight(self, params=None, **kwargs):
        argoparams = params or kwargs
        if (
            "callback" in argoparams
            and type(argoparams["callback"]) == str
            and argoparams["callback"] != ""
        ):
            return self._client.get(
                self._client.api_host(),
                "/ni/advanced/async/json",
                params or kwargs,
                auth_type=NumberInsight.auth_type,
            )
        else:
            raise CallbackRequiredError("A callback is needed for async advanced number insight")



from deprecated import deprecated


@deprecated(
    version='3.0.0',
    reason='Renamed to Application as V1 is out of support and this new \
    naming is in line with other APIs. Please use Application instead.',
)
class ApplicationV2:
    auth_type = 'header'

    def __init__(self, client):
        self._client = client

    def create_application(self, application_data):
        """
        Create an application using the provided `application_data`.

        :param dict application_data: A JSON-style dict describing the application to be created.

        >>> client.application.create_application({ 'name': 'My Cool App!' })

        Details of the `application_data` dict are described at https://developer.vonage.com/api/application.v2#createApplication
        """
        return self._client.post(
            self._client.api_host(),
            "/v2/applications",
            application_data,
            auth_type=ApplicationV2.auth_type,
        )

    def get_application(self, application_id):
        """
        Get application details for the application with `application_id`.

        The format of the returned dict is described at https://developer.vonage.com/api/application.v2#getApplication

        :param str application_id: The application ID.
        :rtype: dict
        """

        return self._client.get(
            self._client.api_host(),
            f"/v2/applications/{application_id}",
            auth_type=ApplicationV2.auth_type,
        )

    def update_application(self, application_id, params):
        """
        Update the application with `application_id` using the values provided in `params`.


        """
        return self._client.put(
            self._client.api_host(),
            f"/v2/applications/{application_id}",
            params,
            auth_type=ApplicationV2.auth_type,
        )

    def delete_application(self, application_id):
        """
        Delete the application with `application_id`.
        """

        self._client.delete(
            self._client.api_host(),
            f"/v2/applications/{application_id}",
            auth_type=ApplicationV2.auth_type,
        )

    def list_applications(self, page_size=None, page=None):
        """
        List all applications for your account.

        Results are paged, so each page will need to be requested to see all applications.

        :param int page_size: The number of items in the page to be returned
        :param int page: The page number of the page to be returned.
        """
        params = _filter_none_values({"page_size": page_size, "page": page})

        return self._client.get(
            self._client.api_host(),
            "/v2/applications",
            params=params,
            auth_type=ApplicationV2.auth_type,
        )


class Application:
    auth_type = 'header'

    def __init__(self, client):
        self._client = client

    def create_application(self, application_data):
        """
        Create an application using the provided `application_data`.

        :param dict application_data: A JSON-style dict describing the application to be created.

        >>> client.application.create_application({ 'name': 'My Cool App!' })

        Details of the `application_data` dict are described at https://developer.vonage.com/api/application.v2#createApplication
        """
        return self._client.post(
            self._client.api_host(),
            "/v2/applications",
            application_data,
            auth_type=Application.auth_type,
        )

    def get_application(self, application_id):
        """
        Get application details for the application with `application_id`.

        The format of the returned dict is described at https://developer.vonage.com/api/application.v2#getApplication

        :param str application_id: The application ID.
        :rtype: dict
        """

        return self._client.get(
            self._client.api_host(),
            f"/v2/applications/{application_id}",
            auth_type=Application.auth_type,
        )

    def update_application(self, application_id, params):
        """
        Update the application with `application_id` using the values provided in `params`.


        """
        return self._client.put(
            self._client.api_host(),
            f"/v2/applications/{application_id}",
            params,
            auth_type=Application.auth_type,
        )

    def delete_application(self, application_id):
        """
        Delete the application with `application_id`.
        """

        self._client.delete(
            self._client.api_host(),
            f"/v2/applications/{application_id}",
            auth_type=Application.auth_type,
        )

    def list_applications(self, page_size=None, page=None):
        """
        List all applications for your account.

        Results are paged, so each page will need to be requested to see all applications.

        :param int page_size: The number of items in the page to be returned
        :param int page: The page number of the page to be returned.
        """
        params = _filter_none_values({"page_size": page_size, "page": page})

        return self._client.get(
            self._client.api_host(),
            "/v2/applications",
            params=params,
            auth_type=Application.auth_type,
        )


def _filter_none_values(d):
    return {k: v for k, v in d.items() if v is not None}



class Verify:
    auth_type = 'params'
    defaults = {'auth_type': auth_type, 'body_is_json': False}

    def __init__(self, client):
        self._client = client

    def start_verification(self, params=None, **kwargs):
        return self._client.post(
            self._client.api_host(),
            "/verify/json",
            params or kwargs,
            **Verify.defaults,
        )

    def check(self, request_id, params=None, **kwargs):
        return self._client.post(
            self._client.api_host(),
            "/verify/check/json",
            dict(params or kwargs, request_id=request_id),
            **Verify.defaults,
        )

    def search(self, request_id):
        return self._client.get(
            self._client.api_host(),
            "/verify/search/json",
            {"request_id": request_id},
            auth_type=Verify.auth_type,
        )

    def cancel(self, request_id):
        return self._client.post(
            self._client.api_host(),
            "/verify/control/json",
            {"request_id": request_id, "cmd": "cancel"},
            **Verify.defaults,
        )

    def trigger_next_event(self, request_id):
        return self._client.post(
            self._client.api_host(),
            "/verify/control/json",
            {"request_id": request_id, "cmd": "trigger_next_event"},
            **Verify.defaults,
        )

    def psd2(self, params=None, **kwargs):
        return self._client.post(
            self._client.api_host(),
            "/verify/psd2/json",
            params or kwargs,
            **Verify.defaults,
        )



from .errors import RedactError

from deprecated import deprecated


@deprecated(
    version='3.0.0',
    reason='This is a dev preview product and as such is not supported in this SDK.',
)
class Redact:
    auth_type = 'header'

    allowed_product_names = {'sms', 'voice', 'number-insight', 'verify', 'verify-sdk', 'messages'}

    def __init__(self, client):
        self._client = client

    def redact_transaction(self, id: str, product: str, type=None):
        self._check_allowed_product_name(product)
        params = {"id": id, "product": product}
        if type is not None:
            params["type"] = type
        return self._client.post(
            self._client.api_host(), "/v1/redact/transaction", params, auth_type=Redact.auth_type
        )

    def _check_allowed_product_name(self, product):
        if product not in self.allowed_product_names:
            raise RedactError(
                f'Invalid product name in redact request. Must be one of {self.allowed_product_names}.'
            )



from pydantic import BaseModel, ValidationInfo, field_validator, validator
from typing import Dict
from typing_extensions import Literal


class PayPrompts:
    class VoicePrompt(BaseModel):
        language: str = None
        style: int = None

    class TextPrompt(BaseModel):
        type: Literal['CardNumber', 'ExpirationDate', 'SecurityCode']
        text: str
        errors: Dict[
            Literal[
                'InvalidCardType',
                'InvalidCardNumber',
                'InvalidExpirationDate',
                'InvalidSecurityCode',
                'Timeout',
            ],
            Dict[Literal['text'], str],
        ]

        @field_validator('errors')
        @classmethod
        def check_valid_error_format(cls, v, info: ValidationInfo):
            values = info.data

            if values['type'] == 'CardNumber':
                allowed_values = {'InvalidCardType', 'InvalidCardNumber', 'Timeout'}
                cls.check_allowed_values(v, allowed_values, values['type'])
            elif values['type'] == 'ExpirationDate':
                allowed_values = {'InvalidExpirationDate', 'Timeout'}
                cls.check_allowed_values(v, allowed_values, values['type'])
            elif values['type'] == 'SecurityCode':
                allowed_values = {'InvalidSecurityCode', 'Timeout'}
                cls.check_allowed_values(v, allowed_values, values['type'])
            return v

        def check_allowed_values(errors, allowed_values, prompt_type):
            for key in errors:
                if key not in allowed_values:
                    raise ValueError(
                        f'Value "{key}" is not a valid error for the "{prompt_type}" prompt type.'
                    )

    @classmethod
    def create_voice_model(cls, dict) -> VoicePrompt:
        return cls.VoicePrompt.model_validate(dict)

    @classmethod
    def create_text_model(cls, dict) -> TextPrompt:
        return cls.TextPrompt.model_validate(dict)



from __future__ import annotations
from typing import TYPE_CHECKING

from vonage.errors import NumberVerificationError

from .camara_auth import CamaraAuth

if TYPE_CHECKING:
    from vonage import Client


class NumberVerification:
    """Class containing methods for working with the Vonage Number Verification API."""

    def __init__(self, client: Client):
        self._client = client
        self._auth_type = 'oauth2'
        self._camara_auth = CamaraAuth(client)
        self._nvtoken = None

    def get_oidc_url(
        self,
        redirect_uri: str,
        state: str = None,
        login_hint: str = None,
        scope: str = 'openid dpv:FraudPreventionAndDetection#number-verification-verify-read',
    ):
        """Get the URL to use for authentication in a front-end application.

        Args:
            redirect_uri (str): The URI to redirect to after authentication.
            scope (str): The scope of the request.
            state (str): A unique identifier for the request. Can be any string.
            login_hint (str): The phone number to use for the request.

        Returns:
            The URL to use to make an OIDC request in a front-end application.
        """
        return self._camara_auth.get_oidc_url(
            redirect_uri=redirect_uri,
            scope=scope,
            state=state,
            login_hint=login_hint,
        )

    def exchange_code_for_token(self, code: str, redirect_uri: str) -> str:
        return self._camara_auth.request_frontend_camara_token(code, redirect_uri)

    def verify(self, access_token: str, phone_number: str = None, hashed_phone_number: str = None):
        """Verify a phone number using the Number Verification API."""

        if phone_number and hashed_phone_number:
            raise NumberVerificationError(
                'Only one of "phone_number" and "hashed_phone_number" can be provided.'
            )
        if phone_number:
            params = {'phoneNumber': phone_number}
        elif hashed_phone_number:
            params = {'hashedPhoneNumber': hashed_phone_number}

        return self._client.post(
            'api-eu.vonage.com',
            '/camara/number-verification/v031/verify',
            params=params,
            auth_type=self._auth_type,
            oauth_token=access_token,
        )
